@@ -1,11 +1,6 @@
-mmer <- function (y, X = NULL, Z = NULL, W = NULL, R = NULL, method = "AI", 
-          REML = TRUE, iters = 40, draw = FALSE, init = NULL, n.PC = 0, 
-          P3D = TRUE, models = "additive", ploidy = 2, min.MAF = 0.05, 
-          silent = FALSE, family = NULL, constraint = TRUE, sherman = FALSE, 
-          MTG2 = FALSE, Fishers = FALSE, gss = TRUE, forced = NULL, 
-          full.rank = TRUE) 
-{
-  if (!is.null(X)) {
+mmer <- function(y, X=NULL, Z=NULL, W=NULL, R=NULL, method="AI", REML=TRUE, iters=40, draw=FALSE, init=NULL, n.PC=0, P3D=TRUE, models="additive", ploidy=2, min.MAF=0.05, silent=FALSE, family=NULL, constraint=TRUE, sherman=FALSE, MTG2=FALSE, Fishers=FALSE, gss=TRUE, forced=NULL, full.rank=TRUE){
+
+  if(!is.null(X)){
     case <- dim(X)[2]
     case2 <- dim(X)[2]
     xor <- X
@@ -14,325 +9,252 @@ mmer <- function (y, X = NULL, Z = NULL, W = NULL, R = NULL, method = "AI",
     X <- as.matrix(X[not.NA, ])
     XtX <- crossprod(X, X)
     rank.X <- qr(XtX)$rank
-    if (rank.X < p & full.rank) {
-      while (rank.X < p) {
+    if(rank.X < p & full.rank){
+      while(rank.X < p){ # remove columns as X is not full matrix
         case <- case - 1
-        XtX <- crossprod(X[, -c(case:case2)], X[, -c(case:case2)])
-        p <- ncol(X[, -c(case:case2)])
+        XtX <- crossprod(X[,-c(case:case2)], X[,-c(case:case2)])
+        p <- ncol(X[,-c(case:case2)])
         rank.X <- qr(XtX)$rank
       }
-      X <- as.matrix(xor[, -c(case:case2)])
+      X <- as.matrix(xor[,-c(case:case2)])
       cat("\nYour X matrix is not full rank, deleting columns until full rank is achieved\n")
-    }
-    else {
+    }else{
       X <- xor
     }
   }
+  ###
   make.full <- function(X) {
     svd.X <- svd(X)
     r <- max(which(svd.X$d > 1e-08))
     return(as.matrix(svd.X$u[, 1:r]))
   }
-  if (is.list(Z)) {
-    if (is.list(Z[[1]])) {
+
+  if(is.list(Z)){
+    if(is.list(Z[[1]])){ 
       provided <- lapply(Z, names)
-      for (s in 1:length(provided)) {
+      for(s in 1:length(provided)){ 
         provided2 <- names(Z[[s]])
-        if (length(provided2) == 1) {
-          if (provided2 == "K") {
-            zz <- diag(length(y))
-            Z[[s]] <- list(Z = zz, K = Z[[s]][[1]])
+        if(length(provided2) ==1){ #----the 's' random effect has one matrix only----
+          if(provided2 == "K"){ #user only provided K
+            zz <- diag(length(y))#model.matrix(~rownames(Z[[s]][[1]]))
+            #colnames(zz) <- rownames(Z[[s]][[1]])
+            Z[[s]] <- list(Z=zz, K=Z[[s]][[1]])
           }
-          if (provided2 == "Z") {
+          if(provided2 == "Z"){ # user only provided Z
             kk <- diag(dim(Z[[s]][[1]])[2])
-            Z[[s]] <- list(Z = Z[[s]][[1]], K = kk)
+            #rownames(kk) <- colnames(Z[[s]][[1]]); colnames(kk) <- rownames(kk)
+            Z[[s]] <- list(Z=Z[[s]][[1]],K=kk) 
           }
-        }
-        else {
-          dido <- lapply(Z[[s]], dim)
-          condi <- (dido$Z[2] == dido$K[1] & dido$Z[2] == 
-                      dido$K[2])
-          if (!condi) {
-            cat(paste("ERROR! In the", s, "th random effect you have provided or created an incidence \nmatrix with dimensions:", 
-                      dido$Z[1], "rows and", dido$Z[2], "columns. Therefore the \nvariance-covariance matrix(K) for this random effect expected was a \nsquare matrix with dimensions", 
-                      dido$Z[2], "x", dido$Z[2]), ", but you provided a", 
-                dido$K[1], "x", dido$K[2], " matrix \nas a variance-covariance matrix. Please double check your matrices.")
+        }else{ #----the 's' random effect has two matrices----
+          dido<-lapply(Z[[s]], dim) # dimensions of Z and K
+          condi<-(dido$Z[2] == dido$K[1] & dido$Z[2] == dido$K[2]) 
+          # condition, column size on Z matches with a square matrix K
+          if(!condi){
+            cat(paste("ERROR! In the",s,"th random effect you have provided or created an incidence \nmatrix with dimensions:",dido$Z[1],"rows and",dido$Z[2],"columns. Therefore the \nvariance-covariance matrix(K) for this random effect expected was a \nsquare matrix with dimensions",dido$Z[2],"x",dido$Z[2]),", but you provided a",dido$K[1],"x",dido$K[2]," matrix \nas a variance-covariance matrix. Please double check your matrices.")
             stop()
           }
-        }
-      }
-    }
-    else {
-      if (length(Z) == 1) {
+        }#---------------------------------------------------------------------------
+      } 
+    }else{ #
+      if(length(Z) == 1){ ## 
         provided <- names(Z)
-        if (provided == "K") {
+        if(provided == "K"){
           zz <- diag(length(y))
-          Z <- list(Z = zz, K = Z[[1]])
+          Z <- list(Z=zz, K=Z[[1]])
         }
-        if (provided == "Z") {
+        if(provided == "Z"){
           kk <- diag(dim(Z[[1]])[2])
-          Z <- list(Z = Z[[1]], K = kk)
+          #rownames(kk) <- colnames(Z[[1]]); colnames(kk) <- rownames(kk)
+          Z <- list(Z=Z[[1]],K=kk) 
         }
-      }
-      else {
-        dido <- lapply(Z, dim)
-        condi <- (dido$Z[2] == dido$K[1] & dido$Z[2] == 
-                    dido$K[2])
-        if (!condi) {
-          cat(paste("ERROR! In the", s, "th random effect you have provided or created an incidence \nmatrix with dimensions:", 
-                    dido$Z[1], "rows and", dido$Z[2], "columns. Therefore the \nvariance-covariance matrix(K) for this random effect expected was a \nsquare matrix with dimensions", 
-                    dido$Z[2], "x", dido$Z[2]), ", but you provided a", 
-              dido$K[1], "x", dido$K[2], " matrix \nas a variance-covariance matrix. Please double check your matrices.")
+      }else{ # 
+        dido<-lapply(Z, dim) # dimensions of Z and K
+        condi<-(dido$Z[2] == dido$K[1] & dido$Z[2] == dido$K[2]) 
+        # condition, column size on Z matches with a square matrix K
+        if(!condi){
+          cat(paste("ERROR! In the",s,"th random effect you have provided or created an incidence \nmatrix with dimensions:",dido$Z[1],"rows and",dido$Z[2],"columns. Therefore the \nvariance-covariance matrix(K) for this random effect expected was a \nsquare matrix with dimensions",dido$Z[2],"x",dido$Z[2]),", but you provided a",dido$K[1],"x",dido$K[2]," matrix \nas a variance-covariance matrix. Please double check your matrices.")
           stop()
-        }
-        else {
-          Z = list(Z = Z)
-        }
+        }else{Z=list(Z=Z)}
       }
     }
-  }
-  else {
+  }else{
+    #stop;
     cat("\nThe parameter 'Z' needs to be provided in a 2-level list structure. \n\nPlease see help typing ?mmer and look at the 'Arguments' section\n")
     cat("\nIf no random effects provided, the model will be fitted using the 'lm' function\n\n")
   }
-  Z <- lapply(Z, function(x) {
-    if (length(x) > 1) {
-      if (length(x) > 1 | !is.diagonal.matrix(x[[2]])) {
-        if (!is.null(colnames(x[[1]])) & !is.null(colnames(x[[2]]))) {
-          if (length(which(colnames(x[[1]]) == colnames(x[[2]]))) != 
-              dim(x[[1]])[2]) {
+
+  
+  Z <- lapply(Z, function(x){ 
+    ## 
+    if(length(x) > 1){
+      if(length(x) > 1 | !is.diagonal.matrix(x[[2]])){ #do something
+        if(!is.null(colnames(x[[1]])) & !is.null(colnames(x[[2]]))){ #
+          if(length(which(colnames(x[[1]]) == colnames(x[[2]]))) != dim(x[[1]])[2]){
             y <- colnames(x[[1]])
-            y2 <- strsplit(y, split = "")
+            y2 <- strsplit(y, split="")
             y3 <- y2[[1]]
-            for (i in 2:length(y2)) {
+            
+            
+            for(i in 2:length(y2)){ # find the common name added when creating the design matrix
+              #y3 <- y3[match(y3, y2[[i]])]
               basd <- vector()
-              for (j in 1:length(y3)) {
+              for(j in 1:length(y3)){ # for each letter
                 good <- which(y3[j] == y2[[i]])
-                if (length(good) > 0) {
+                if(length(good) > 0){
                   basd[j] <- (y3[good])[1]
                   y2[[i]][good] <- NA
                 }
               }
               y3 <- basd
-            }
+            }# 
+            ## &&&&&&&&&&&&&&&&&&&
             extraname <- paste(na.omit(y3), collapse = "")
-            if (extraname != "") {
-              real1 <- match(colnames(x[[2]]), gsub(as.character(extraname), 
-                                                    "", as.character(colnames(x[[1]]))))
-              if (length(which(is.na(real1))) == 0) {
-                x[[1]] <- x[[1]][, real1]
-              }
-              else {
+            if(extraname != ""){ #
+              
+              real1 <- match(  colnames(x[[2]]), gsub(as.character(extraname),"",as.character(colnames(x[[1]]))))
+              
+              if(length(which(is.na(real1))) == 0){ # just fix it if all levels were there, 
+                x[[1]] <- x[[1]][,real1] # reorder Z !!!!! and put it back
+              }else{ # otherwise do not fix it but let the user know that Z colnames should be the same than K colnames
                 cat("\nWe found the column names of your Z matrix in different order than column names of K, \nwe were not able to fix it. The analysis will be performed but this could lead to wrong \nresults please take a look to make sure the levels of K and Z are in the same order.\n")
-              }
-            }
-          }
+              }#
+              
+            } 
+            
+          }#
         }
-      }
+      } # )
     }
     return(x)
   })
-  Z <- lapply(Z, function(x) {
-    im <- x[[1]]
-    im <- apply(im, 2, function(y) {
-      qq <- which(is.na(y))
-      if (length(qq) > 0) {
-        y[qq] <- median(y, na.rm = TRUE)
-      }
-      return(y)
-    })
-    z = list(Z = im, K = x[[2]])
+
+
+  
+  Z <- lapply(Z, function(x){
+    im <- x[[1]] 
+    im <- apply(im,2,function(y){qq <- which(is.na(y)); if(length(qq)>0){y[qq] <- median(y,na.rm=TRUE)}; return(y)})
+    z=list(Z=im,K=x[[2]])
     return(z)
   })
-  if (is.list(Z)) {
-    if (!is.null(Z) & !is.null(W)) {
-      y[which(is.na(y))] <- mean(y, na.rm = TRUE)
+ 
+  
+  if(is.list(Z)){
+    if(!is.null(Z) & !is.null(W)){
+      #
+      y[which(is.na(y))] <- mean(y, na.rm=TRUE)
+      ####
       misso <- which(is.na(y))
-      if (length(misso) > 0) {
-        y[misso] <- mean(y, na.rm = TRUE)
-      }
-      if (is.null(colnames(W))) {
-        colnames(W) <- paste("M", 1:dim(W)[2], sep = "-")
-      }
-      W <- apply(W, 2, function(x) {
-        vv <- which(is.na(x))
-        if (length(vv) > 0) {
-          mu <- mean(x, na.rm = TRUE)
-          x[vv] <- mu
-        }
-        else {
-          x <- x
-        }
-      })
+      if(length(misso)>0){y[misso] <- mean(y,na.rm=TRUE)}
+      #Wcnames <- colnames(W); Wrnames <- rownames(W)
+      if(is.null(colnames(W))){colnames(W) <- paste("M",1:dim(W)[2],sep="-")}
+      W <- apply(W, 2, function(x){vv<- which(is.na(x)); if(length(vv) > 0){mu<- mean(x, na.rm = TRUE); x[vv] <- mu}else{x<-x}})
+      #colnames(W) <- Wcnames;rownames(W) <- Wrnames
       cat("Estimating variance components\n")
-      fixed <- which(unlist(lapply(Z, function(x) {
-        names(x)[1]
-      })) == "X")
-      random <- which(unlist(lapply(Z, function(x) {
-        names(x)[1]
-      })) == "Z")
+      fixed <- which(unlist(lapply(Z, function(x){names(x)[1]})) == "X") # elements of Z that are FIXED
+      random <- which(unlist(lapply(Z, function(x){names(x)[1]})) == "Z") # elements of Z that are RANDOM
       random2 <- which(names(Z) == "Z")
+      ## 
+      
       if (n.PC > 0) {
         KK <- A.mat(W, shrink = FALSE)
         eig.vec <- eigen(KK)$vectors
-        if (is.null(X)) {
-          X <- as.matrix(rep(1, dim(KK)[1]))
-        }
+        if(is.null(X)){X <- as.matrix(rep(1,dim(KK)[1]))}
         ZZ.comp <- list()
-        for (i in 1:length(Z)) {
-          X <- cbind(X, Z[[i]][[1]] %*% (1 + as.matrix(eig.vec[, 
-                                                               1:n.PC])))
+        for(i in 1:length(Z)){
+          X <- cbind(X, Z[[i]][[1]] %*% (1 + as.matrix(eig.vec[,1:n.PC]))) #plus 1 to ensure X is positive definite
         }
         X <- make.full(X)
       }
-      if (length(random) > 1 & method == "EMMA") {
-        stop
-        cat("\nError, The EMMA and GEMMA method were design to only deal with a single variance component other than error, please select method='AI' or method='EM' which can estimate more than one variance component\n\n")
+      ## 
+      if(length(random) > 1 & method == "EMMA"){
+        stop;cat("\nError, The EMMA and GEMMA method were design to only deal with a single variance component other than error, please select method='AI' or method='EM' which can estimate more than one variance component\n\n")
       }
-      if ((length(random) == 1 | length(random2) == 1) & 
-          method == "EMMA") {
-        if (length(random) > 0) {
-          res <- EMMA(y = y, X = X, Z = Z[[random]][[1]], 
-                      K = Z[[random]][[2]], REML = REML)
-        }
-        else {
-          res <- EMMA(y = y, X = X, Z = Z[[1]], K = Z[[2]], 
-                      REML = REML)
+      if((length(random) == 1 | length(random2) == 1) & method == "EMMA"){
+        if(length(random) > 0){# E
+          res <- EMMA(y=y, X=X, Z=Z[[random]][[1]], K=Z[[random]][[2]],  REML=REML) 
+        }else{
+          res <- EMMA(y=y, X=X, Z=Z[[1]], K=Z[[2]],REML=REML) 
         }
       }
-      if (method == "EM") {
-        res <- EM(y = y, X = X, ETA = Z, R = R, init = init, 
-                  iters = iters, REML = REML, draw = draw, silent = silent, 
-                  forced = forced)
+      if(method == "EM"){
+        res <- EM(y=y, X=X, ETA=Z, R=R, init=init, iters = iters, REML=REML, draw=draw, silent=silent, forced=forced)
       }
-      if (method == "AI") {
-        if (length(Z) == 1) {
-          dias <- unlist(lapply(Z[[1]], function(x) {
-            if (dim(x)[1] == dim(x)[2]) {
-              y <- is.diagonal.matrix(x)
-            }
-            else {
-              y <- FALSE
-            }
-            return(y)
-          }))
-          if (length(which(dias)) == 2) {
-            res <- EMMA(y = y, X = X, Z = Z[[random]][[1]], 
-                        K = Z[[random]][[2]], REML = REML)
+      if(method == "AI"){
+        #res <- AI(y=y, X=X, ZETA=Z, R=R, REML=REML, draw=draw, silent=silent, iters = iters, constraint=constraint, init=init, sherman=sherman, che=FALSE, MTG2=MTG2,Fishers=Fishers)
+        if(length(Z) == 1){ # 
+          dias <- unlist(lapply(Z[[1]], function(x){if(dim(x)[1] == dim(x)[2]){y <- is.diagonal.matrix(x)}else{y <- FALSE};return(y)}))
+          if(length(which(dias)) == 2){ # 
+            res <- EMMA(y=y, X=X, Z=Z[[random]][[1]], K=Z[[random]][[2]],REML=REML)
+          }else{
+            res <- AI(y=y, X=X, ZETA=Z, R=R, REML=REML, draw=draw, silent=silent, iters = iters, constraint=constraint, init=init, sherman=sherman, che=FALSE, MTG2=MTG2,Fishers=Fishers, forced=forced)
           }
-          else {
-            res <- AI(y = y, X = X, ZETA = Z, R = R, 
-                      REML = REML, draw = draw, silent = silent, 
-                      iters = iters, constraint = constraint, 
-                      init = init, sherman = sherman, che = FALSE, 
-                      MTG2 = MTG2, Fishers = Fishers, forced = forced)
-          }
-        }
-        else {
-          res <- AI(y = y, X = X, ZETA = Z, R = R, REML = REML, 
-                    draw = draw, silent = silent, iters = iters, 
-                    constraint = constraint, init = init, sherman = sherman, 
-                    che = FALSE, MTG2 = MTG2, Fishers = Fishers, 
-                    forced = forced)
+        }else{ # 
+          res <- AI(y=y, X=X, ZETA=Z, R=R, REML=REML, draw=draw, silent=silent, iters = iters, constraint=constraint, init=init, sherman=sherman, che=FALSE, MTG2=MTG2,Fishers=Fishers,forced=forced) 
         }
       }
-      if (n.PC > 0) {
-        X2 <- X
-      }
-      else {
-        X2 <- make.full(res$X)
-      }
-      min.MAF = min.MAF
-      n <- length(y)
+ 
+      
+      if(n.PC > 0){X2 <- X}else{ X2 <- make.full(res$X)}
+      min.MAF = min.MAF; n <- length(y)
+ 
+      
       cat("\nPerforming GWAS")
-      max.geno.freq = 1 - min.MAF
+      max.geno.freq= 1 - min.MAF
+
+      
       W.scores <- list(NA)
-      if (length(models) > 2) {
-        layout(matrix(1:4, 2, 2))
-      }
-      else {
-        layout(matrix(1:length(models), 1, length(models)))
-      }
-      for (u in 1:length(models)) {
+      # draw layout
+      if(length(models) > 2){layout(matrix(1:4,2,2))}else{layout(matrix(1:length(models),1,length(models)))}
+      # run
+      for(u in 1:length(models)){ #### #
         model <- models[u]
-        cat(paste("\nRunning", model, "model"))
+        cat(paste("\nRunning",model,"model"))
         ZO <- diag(dim(W)[1])
-        step2 <- score.calc(marks = colnames(W), y = y, 
-                            Z = ZO, X = X2, K = res$K, M = W, Hinv = res$V.inv, 
-                            ploidy = ploidy, model = model, min.MAF = min.MAF, 
-                            max.geno.freq = max.geno.freq)
+        step2 <- score.calc(marks=colnames(W),y=y,Z=ZO,X=X2,K=res$K,M=W,Hinv=res$V.inv,ploidy=ploidy,model=model,min.MAF=min.MAF,max.geno.freq=max.geno.freq)
         W.scores[[u]] <- as.matrix(step2$score)
         rownames(W.scores[[u]]) <- colnames(W)
-        plot(step2$score, col = transp("cadetblue", 0.6), 
-             pch = 20, xlab = "Marker index", ylab = "-log10(p)", 
-             main = paste(model, "model"), bty = "n", cex = 1.5)
+        plot(step2$score, col=transp("cadetblue", 0.6), pch=20, xlab="Marker index", ylab="-log10(p)", main=paste(model,"model"), bty="n", cex=1.5)
       }
       names(W.scores) <- models
+
+      
       res$W.scores <- W.scores
       res$W <- W
       res$method <- method
-    }
+    } # end of GWAS 
   }
-  if (is.list(Z)) {
-    if ((!is.null(Z) & is.null(W))) {
+
+  
+  if(is.list(Z)){
+    if((!is.null(Z) & is.null(W)) ) {
       cat("Estimating variance components\n")
-      fixed <- which(unlist(lapply(Z, function(x) {
-        names(x)[1]
-      })) == "X")
-      random <- which(unlist(lapply(Z, function(x) {
-        names(x)[1]
-      })) == "Z")
+      fixed <- which(unlist(lapply(Z, function(x){names(x)[1]})) == "X") # elem
+      random <- which(unlist(lapply(Z, function(x){names(x)[1]})) == "Z") # el
       random2 <- which(names(Z) == "Z")
-      if (length(random) > 1 & method == "EMMA") {
-        stop
-        cat("\nError, The EMMA and GEMMA method were design to only deal with a single variance component other than error, please select method='AI' or method='EM' which can estimate more than one variance component\n\n")
+      ## 
+      if(length(random) > 1 & method == "EMMA"){
+        stop;cat("\nError, The EMMA and GEMMA method were design to only deal with a single variance component other than error, please select method='AI' or method='EM' which can estimate more than one variance component\n\n")
       }
-      if ((length(random) == 1 | length(random2) == 1) & 
-          method == "EMMA") {
-        if (length(random) > 0) {
-          res <- EMMA(y = y, X = X, Z = Z[[random]][[1]], 
-                      K = Z[[random]][[2]], REML = REML)
-        }
-        else {
-          res <- EMMA(y = y, X = X, Z = Z[[1]], K = Z[[2]], 
-                      REML = REML)
+      if((length(random) == 1 | length(random2) == 1) & method == "EMMA"){
+        if(length(random) > 0){#
+          res <- EMMA(y=y, X=X, Z=Z[[random]][[1]], K=Z[[random]][[2]],REML=REML) 
+        }else{
+          res <- EMMA(y=y, X=X, Z=Z[[1]], K=Z[[2]], REML=REML) 
         }
       }
-      if (method == "EM") {
-        res <- EM(y = y, X = X, ETA = Z, R = R, init = init, 
-                  iters = iters, REML = REML, draw = draw, silent = silent, 
-                  forced = forced)
+      if(method == "EM"){
+        res <- EM(y=y, X=X, ETA=Z, R=R, init=init, iters = iters, REML=REML, draw=draw, silent=silent, forced=forced)
       }
-      if (method == "AI") {
-        if (length(Z) == 1) {
-          dias <- unlist(lapply(Z[[1]], function(x) {
-            if (dim(x)[1] == dim(x)[2]) {
-              y <- is.diagonal.matrix(x)
-            }
-            else {
-              y <- FALSE
-            }
-            return(y)
-          }))
-          if (length(which(dias)) == 2) {
-            res <- EMMA(y = y, X = X, Z = Z[[random]][[1]], 
-                        K = Z[[random]][[2]], REML = REML)
+      if(method == "AI"){
+        if(length(Z) == 1){ # 
+          dias <- unlist(lapply(Z[[1]], function(x){if(dim(x)[1] == dim(x)[2]){y <- is.diagonal.matrix(x)}else{y <- FALSE};return(y)}))
+          if(length(which(dias)) == 2){ # if K and Z are diagonals do EMMA
+            res <- EMMA(y=y, X=X, Z=Z[[random]][[1]], K=Z[[random]][[2]],REML=REML)
+          }else{
+            res <- AI(y=y, X=X, ZETA=Z, R=R, REML=REML, draw=draw, silent=silent, iters = iters, constraint=constraint, init=init, sherman=sherman, che=FALSE, MTG2=MTG2,Fishers=Fishers, gss=gss, forced=forced)
           }
-          else {
-            res <- AI(y = y, X = X, ZETA = Z, R = R, 
-                      REML = REML, draw = draw, silent = silent, 
-                      iters = iters, constraint = constraint, 
-                      init = init, sherman = sherman, che = FALSE, 
-                      MTG2 = MTG2, Fishers = Fishers, gss = gss, 
-                      forced = forced)
-          }
-        }
-        else {
-          res <- AI(y = y, X = X, ZETA = Z, R = R, REML = REML, 
-                    draw = draw, silent = silent, iters = iters, 
-                    constraint = constraint, init = init, sherman = sherman, 
-                    che = FALSE, MTG2 = MTG2, Fishers = Fishers, 
-                    gss = gss, forced = forced)
+        }else{ # 
+          res <- AI(y=y, X=X, ZETA=Z, R=R, REML=REML, draw=draw, silent=silent, iters = iters, constraint=constraint, init=init, sherman=sherman, che=FALSE, MTG2=MTG2,Fishers=Fishers, gss=gss, forced=forced) 
         }
       }
       res$method <- method
@@ -340,19 +262,21 @@ mmer <- function (y, X = NULL, Z = NULL, W = NULL, R = NULL, method = "AI",
       res$W <- W
     }
   }
-  if ((is.null(X) & is.null(Z) & is.null(W))) {
-    res <- lm(y ~ 1)
+
+  
+  if((is.null(X) & is.null(Z) & is.null(W)) ) {
+    res <- lm(y~1)
   }
-  if ((!is.null(X) & is.null(Z) & is.null(W))) {
-    res <- lm(y ~ X - 1)
+  if((!is.null(X) & is.null(Z) & is.null(W)) ) {
+    res <- lm(y~ X-1)
   }
-  class(res) <- c("mmer")
-  layout(matrix(1, 1, 1))
+  class(res)<-c("mmer")
+  #res$u.hat <- lapply(res$u.hat, function(x,y){rownames(x) <- colnames(y); return(x)}, y=Z)
+  layout(matrix(1,1,1))
   return(res)
 }
 
-#### =========== ####
-## SUMMARY FUNCTION #
+
 #### =========== ####
 "summary.mmer" <- function(object, ...) {
   digits = max(3, getOption("digits") - 3)
@@ -389,7 +313,7 @@ mmer <- function (y, X = NULL, Z = NULL, W = NULL, R = NULL, method = "AI",
   if(dim(coef)[1] == 1){rownames(coef) <- "Intercept"}
   #printCoefmat((coef))
   #cat("=======================================================")
-  #cat("\nVar-Cov for Fixed effects:\n")
+  #cat("\nVar-Cov for Fixed effects (diagonals are variances):\n")
   varbhat <- data.frame(as.matrix(object$Var.beta.hat))
   rownames(varbhat) <- colnames(object$X)
   colnames(varbhat) <- rownames(varbhat) 
@@ -410,7 +334,7 @@ mmer <- function (y, X = NULL, Z = NULL, W = NULL, R = NULL, method = "AI",
   cat("\nInformation contained in this fitted model: \n* Variance components\n* Residuals and conditional residuals\n* BLUEs and BLUPs\n* Inverse phenotypic variance(V)\n* Variance-covariance matrix for fixed effects\n* Variance-covariance matrix for random effects\n* Predicted error variance (PEV)\n* LogLikelihood\n* AIC and BIC\n* Fitted values\nUse the 'str' function to access such information\n")
   cat("\n=======================================================")
   cat("\nLinear mixed model fit by restricted maximum likelihood\n")
-  cat("********************  sommer 1.4  *********************\n")
+  cat("********************  sommer 1.5  *********************\n")
   cat("=======================================================")
   cat("\nMethod:")
   print(x$method)
@@ -439,7 +363,7 @@ mmer <- function (y, X = NULL, Z = NULL, W = NULL, R = NULL, method = "AI",
   #if(dim(coef)[1] == 1){rownames(coef) <- "Intercept"}
   printCoefmat((x$coef))
   cat("=======================================================")
-  cat("\nVar-Cov for Fixed effects:\n")
+  cat("\nVar-Cov for Fixed effects:\n(diagonals are variances)\n")
   #varbhat <- data.frame(x$Var.beta.hat)
   #colnames(varbhat) <- rownames(varbhat) 
   #if(dim(varbhat)[1] == 1){rownames(varbhat) <- "Intercept"}
@@ -448,8 +372,7 @@ mmer <- function (y, X = NULL, Z = NULL, W = NULL, R = NULL, method = "AI",
   cat("\nUse the 'str' function to have access to all information\n\n")
 }
 
-#### =========== ######
-## RESIDUALS FUNCTION #
+
 #### =========== ######
 "residuals.mmer" <- function(object, type="conditional", ...) {
   digits = max(3, getOption("digits") - 3)
@@ -465,9 +388,7 @@ mmer <- function (y, X = NULL, Z = NULL, W = NULL, R = NULL, method = "AI",
   print((x))
 }
 
-#### =========== ######
-## RANEF FUNCTION #
-#### =========== ######
+
 
 "randef" <- function(object) {
   digits = max(3, getOption("digits") - 3)
@@ -479,9 +400,8 @@ mmer <- function (y, X = NULL, Z = NULL, W = NULL, R = NULL, method = "AI",
 #  print((x))
 #}
 
-#### =========== ######
-## FIXEF FUNCTION #
-#### =========== ######
+
+
 "fixef.mmer" <- function(object, ...) {
   digits = max(3, getOption("digits") - 3)
   output <- object$beta.hat
@@ -492,9 +412,7 @@ mmer <- function (y, X = NULL, Z = NULL, W = NULL, R = NULL, method = "AI",
   print((x))
 }
 
-#### =========== ####
-## FITTED FUNCTION ##
-#### =========== ####
+
 "fitted.mmer" <- function(object, type="complete", ...) {
   #type="complete" 
   digits = max(3, getOption("digits") - 3)
@@ -509,9 +427,7 @@ mmer <- function (y, X = NULL, Z = NULL, W = NULL, R = NULL, method = "AI",
   print((x))
 } 
 
-#### =========== ####
-## COEF FUNCTION ####
-#### =========== ####
+
 "coef.mmer" <- function(object, ...){
   object$beta.hat
 }
@@ -520,9 +436,7 @@ mmer <- function (y, X = NULL, Z = NULL, W = NULL, R = NULL, method = "AI",
   print((x))
 } 
 
-#### =========== ####
-## ANOVA FUNCTION ###
-#### =========== ####
+
 anova.mmer <- function(object, object2=NULL, ...) {
   signifo <- function(x){
     if(x >= 0 & x < 0.001){y="***"}
@@ -572,9 +486,7 @@ anova.mmer <- function(object, object2=NULL, ...) {
   }
 }
 
-#### =========== ####
-## PLOTING FUNCTION #
-#### =========== ####
+
 plot.mmer <- function(x, ...) {
   digits = max(3, getOption("digits") - 3)
   transp <- function (col, alpha = 0.5){
@@ -592,10 +504,6 @@ plot.mmer <- function(x, ...) {
   layout(matrix(1,1,1))
 }
 
-
-##################################################################################################
-#Startup function
-#this function is executed once the library is loaded
 .onAttach = function(library, pkg)
 {
   Rv = R.Version()
@@ -603,7 +511,7 @@ plot.mmer <- function(x, ...) {
     stop("This package requires R 2.1 or later")
   assign(".sommer.home", file.path(library, pkg),
          pos=match("package:sommer", search()))
-  sommer.version = "1.4 (2016-04-15)"
+  sommer.version = "1.5 (2016-05-01)"
   assign(".sommer.version", sommer.version, pos=match("package:sommer", search()))
   if(interactive())
   {
